@@ -32,9 +32,8 @@ enum
 /* account flags */
 #define M_ACCT_PORT (1<<0)
 #define M_ACCT_USER (1<<1)
-#define M_ACCT_SSL  (1<<2)
-#define M_ACCT_CRAM (1<<3)
-#define M_ACCT_PASS (1<<4)
+#define M_ACCT_PASS (1<<2)
+#define M_ACCT_SSL  (1<<3)
 
 typedef struct
 {
@@ -46,7 +45,8 @@ typedef struct
   unsigned char flags;
 } ACCOUNT;
 
-/* imap_account_match: compare account info (host/port/user) */
 int mutt_account_match (const ACCOUNT* a1, const ACCOUNT* m2);
+int mutt_account_getuser (ACCOUNT* account);
+int mutt_account_getpass (ACCOUNT* account);
 
 #endif /* _MUTT_ACCOUNT_H_ */
