@@ -277,6 +277,60 @@ int _mutt_enter_string (unsigned char *buf, size_t buflen, int y, int x,
 	    }
 	  }
 	  break;
+	case OP_EDITOR_BACKWARD_WORD:
+	  if (curpos == 0)
+	  {
+	    BEEP ();
+	  }
+	  else
+	  {
+	    if (curpos > 0 && !ISSPACE (buf[curpos]) && ISSPACE (buf[curpos - 1]))
+	      curpos--;
+	    while (curpos > 0 && ISSPACE (buf[curpos]))
+	      curpos--;
+	    while (curpos > 0 && !ISSPACE (buf[curpos]))
+	      curpos--;
+	    if (ISSPACE (buf[curpos]) && !ISSPACE (buf[curpos + 1]))
+	      curpos++;
+
+	    if (!pass)
+	    {
+	      if (curpos < begin)
+	      {
+		begin = curpos - width/2;
+		redraw = M_REDRAW_LINE;
+	      }
+	      else
+		move (y, x + curpos - begin);
+	    }
+	  }
+	  break;
+
+	case OP_EDITOR_FORWARD_WORD:
+	  if (curpos == lastchar)
+	  {
+	    BEEP ();
+	  }
+	  else
+	  {
+	    while (curpos < lastchar && ISSPACE (buf[curpos]))
+	      curpos++;
+	    while (curpos < lastchar && !ISSPACE (buf[curpos]))
+	      curpos++;
+
+	    if (!pass)
+	    {
+	      if (curpos >= begin + width)
+	      {
+		begin = curpos - width / 2;
+		redraw = M_REDRAW_LINE;
+	      }
+	      else
+		move (y, x + curpos - begin);
+	    }
+	  }
+	  break;
+
 	case OP_EDITOR_DELETE_CHAR:
 	  if (curpos != lastchar)
 	  {
@@ -324,6 +378,19 @@ int _mutt_enter_string (unsigned char *buf, size_t buflen, int y, int x,
 	    }
 	  }
 	  break;
+
+	case OP_EDITOR_KILL_EOW:
+	  /* delete to end of word */
+	  for (j = curpos; j < lastchar && ISSPACE (buf[j]); j++)
+	    ;
+	  for (          ; j < lastchar && !ISSPACE (buf[j]); j++)
+	    ;
+	  for (ch = curpos; j < lastchar; j++, ch++)
+	    buf[ch] = buf[j];
+	  lastchar = ch;
+	  redraw = M_REDRAW_EOL;
+	  break;
+
 	case OP_EDITOR_BUFFY_CYCLE:
 	  if (flags & M_EFILE)
 	  {
@@ -484,6 +551,29 @@ int _mutt_enter_string (unsigned char *buf, size_t buflen, int y, int x,
 	    goto self_insert;
 	  }
 
+	case OP_EDITOR_TRANSPOSE_CHARS:
+	  j = buf[curpos];
+	  if(curpos == 0) 
+	  {
+	    buf[curpos] = buf[1];
+	    buf[1] = j;
+	  }
+	  else if (curpos == lastchar) 
+	  {
+	    j = buf[curpos-1];
+	    buf[curpos-1] = buf[curpos-2];
+	    buf[curpos-2] = j;
+	  }
+	  else 
+	  {
+	    buf[curpos] = buf[curpos-1];
+	    buf[curpos-1] = j;
+	  }
+	  if (curpos < lastchar)
+	    curpos++;
+	  redraw = M_REDRAW_LINE;
+	  break;
+
 	default:
 	  BEEP ();
       }
