@@ -19,10 +19,12 @@
 #ifndef _MUTT_SSL_H_
 #define _MUTT_SSL_H_ 1
 
+#include "mutt_socket.h"
+
 extern char *SslCertFile;
 extern char *SslEntropyFile;
 
 extern int ssl_socket_setup (CONNECTION *conn);
-extern void imap_set_ssl (IMAP_MBOX *mx);
+extern void imap_set_ssl (ACCOUNT* account);
 
-#endif
+#endif /* _MUTT_SSL_H_ */
