@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -193,13 +193,13 @@ const char *mutt_attach_fmt (char *dest,
       {
 	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
 	if (mutt_is_text_type (aptr->content->type, aptr->content->subtype) &&
-	    mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+	    mutt_get_body_charset (charset, sizeof (charset), aptr->content))
 	  snprintf (dest, destlen, fmt, charset);
 	else
 	  snprintf (dest, destlen, fmt, "");
       }
       else if (!mutt_is_text_type (aptr->content->type, aptr->content->subtype) ||
-	       !mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+	       !mutt_get_body_charset (charset, sizeof (charset), aptr->content))
         optional = 0;
       break;
     case 'c':
@@ -484,7 +484,7 @@ void mutt_save_attachment_list (FILE *fp, int tag, BODY *top, HEADER *hdr)
   }
 
   if (!option (OPTATTACHSPLIT) && (rc == 0))
-    mutt_message _("Attachment saved");
+    mutt_message _("Attachment saved.");
 }
 
 static void
@@ -795,6 +795,8 @@ void mutt_view_attachments (HEADER *hdr)
   /* make sure we have parsed this message */
   mutt_parse_mime_message (Context, hdr);
 
+  mutt_message_hook (Context, hdr, M_MESSAGEHOOK);
+  
   if ((msg = mx_open_message (Context, hdr->msgno)) == NULL)
     return;
 
